@@ -4,22 +4,18 @@
 #include "jdupes.h"
 
 /* Compile out the code if no linking support is built in */
-#if !(defined NO_HARDLINKS && defined NO_SYMLINKS)
+#if !(defined NO_HARDLINKS && !defined NO_SYMLINKS && !defined ENABLE_DEDUPE)
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 
+#include <libjodycode.h>
 #include "act_linkfiles.h"
-#include "jody_win_unicode.h"
-#include "oom.h"
-#ifdef ON_WINDOWS
- #include "win_stat.h"
-#endif
 
 #ifdef UNICODE
- wpath_t wname, wname2;
+ static wpath_t wname, wname2;
 #endif
 
 /* Apple clonefile() is basically a hard link */
@@ -37,14 +33,57 @@
  #endif /* __APPLE__ */
 #endif /* ENABLE_DEDUPE */
 
+
+#ifdef ENABLE_CLONEFILE_LINK
+static void clonefile_error(const char * const restrict func, const char * const restrict name)
+{
+  fprintf(stderr, "warning: %s failed for destination file, reverting:\n-##-> ", func);
+  jc_fwprint(stderr,name, 1);
+  exit_status = EXIT_FAILURE;
+  return;
+}
+#endif /* ENABLE_CLONEFILE_LINK */
+
+
+/* Only build this function if some functionality does not exist */
+#if defined NO_SYMLINKS || defined NO_HARDLINKS || !defined ENABLE_CLONEFILE_LINK
+static void linkfiles_nosupport(const char * const restrict call, const char * const restrict type)
+{
+  fprintf(stderr, "internal error: linkfiles(%s) called without %s support\nPlease report this to the author as a program bug\n", call, type);
+  exit(EXIT_FAILURE);
+}
+#endif /* anything unsupported */
+
+
+#ifdef ON_WINDOWS
+static void mb2wc_failed(const char * const restrict name)
+{
+  fprintf(stderr, "error: MultiByteToWideChar failed: ");
+  jc_fwprint(stderr, name, 1);
+  exit_status = EXIT_FAILURE;
+  return;
+}
+#endif /* ON_WINDOWS */
+
+
+static void revert_failed(const char * const restrict orig, const char * const restrict current)
+{
+  fprintf(stderr, "\nwarning: couldn't revert the file to its original name\n");
+  fprintf(stderr, "original: "); jc_fwprint(stderr, orig, 1);
+  fprintf(stderr, "current:  "); jc_fwprint(stderr, current, 1);
+  exit_status = EXIT_FAILURE;
+  return;
+}
+
+
 /* linktype: 0=symlink, 1=hardlink, 2=clonefile() */
-extern void linkfiles(file_t *files, const int linktype, const int only_current)
+void linkfiles(file_t *files, const int linktype, const int only_current)
 {
   static file_t *tmpfile;
   static file_t *srcfile;
   static file_t *curfile;
   static file_t ** restrict dupelist;
-  static unsigned int counter;
+  static unsigned int counter = 0;
   static unsigned int max = 0;
   static unsigned int x = 0;
   static size_t name_len = 0;
@@ -63,6 +102,7 @@ extern void linkfiles(file_t *files, const int linktype, const int only_current)
   LOUD(fprintf(stderr, "linkfiles(%d): %p\n", linktype, files);)
   curfile = files;
 
+  /* Calculate a maximum */
   while (curfile) {
     if (ISFLAG(curfile->flags, FF_HAS_DUPES)) {
       counter = 1;
@@ -81,7 +121,7 @@ extern void linkfiles(file_t *files, const int linktype, const int only_current)
 
   dupelist = (file_t**) malloc(sizeof(file_t*) * max);
 
-  if (!dupelist) oom("linkfiles() dupelist");
+  if (!dupelist) jc_oom("linkfiles() dupelist");
 
   while (files) {
     if (ISFLAG(files->flags, FF_HAS_DUPES)) {
@@ -98,13 +138,12 @@ extern void linkfiles(file_t *files, const int linktype, const int only_current)
 
       /* Link every file to the first file */
 
-      if (linktype) {
+      if (linktype != 0) {
 #ifndef NO_HARDLINKS
         x = 2;
         srcfile = dupelist[1];
 #else
-        fprintf(stderr, "internal error: linkfiles(hard) called without hard link support\nPlease report this to the author as a program bug\n");
-        exit(EXIT_FAILURE);
+        linkfiles_nosupport("hard", "hard link");
 #endif
       } else {
 #ifndef NO_SYMLINKS
@@ -120,18 +159,18 @@ extern void linkfiles(file_t *files, const int linktype, const int only_current)
         /* If no normal file exists, abort */
         if (srcfile == NULL) continue;
 #else
-        fprintf(stderr, "internal error: linkfiles(soft) called without symlink support\nPlease report this to the author as a program bug\n");
-        exit(EXIT_FAILURE);
+        linkfiles_nosupport("soft", "symlink");
 #endif
       }
       if (!ISFLAG(flags, F_HIDEPROGRESS)) {
-        printf("[SRC] "); fwprint(stdout, srcfile->d_name, 1);
+        printf("[SRC] "); jc_fwprint(stdout, srcfile->d_name, 1);
       }
-#ifdef ENABLE_CLONEFILE_LINK
       if (linktype == 2) {
+#ifdef ENABLE_CLONEFILE_LINK
         if (STAT(srcfile->d_name, &s) != 0) {
           fprintf(stderr, "warning: stat() on source file failed, skipping:\n[SRC] ");
-          fwprint(stderr, srcfile->d_name, 1);
+          jc_fwprint(stderr, srcfile->d_name, 1);
+          exit_status = EXIT_FAILURE;
           continue;
         }
 
@@ -139,14 +178,17 @@ extern void linkfiles(file_t *files, const int linktype, const int only_current)
          * (which can result in files being unreadable), so we want to retain
          * the compression flag of srcfile */
         srcfile_preserved_flags = s.st_flags & UF_COMPRESSED;
-      }
+#else
+        linkfiles_nosupport("clone", "clonefile");
 #endif
+      }
       for (; x <= counter; x++) {
         if (linktype == 1 || linktype == 2) {
           /* Can't hard link files on different devices */
           if (srcfile->device != dupelist[x]->device) {
             fprintf(stderr, "warning: hard link target on different device, not linking:\n-//-> ");
-            fwprint(stderr, dupelist[x]->d_name, 1);
+            jc_fwprint(stderr, dupelist[x]->d_name, 1);
+            exit_status = EXIT_FAILURE;
             continue;
           } else {
             /* The devices for the files are the same, but we still need to skip
@@ -155,7 +197,7 @@ extern void linkfiles(file_t *files, const int linktype, const int only_current)
               /* Don't show == arrows when not matching against other hard links */
               if (ISFLAG(flags, F_CONSIDERHARDLINKS))
                 if (!ISFLAG(flags, F_HIDEPROGRESS)) {
-                  printf("-==-> "); fwprint(stdout, dupelist[x]->d_name, 1);
+                  printf("-==-> "); jc_fwprint(stdout, dupelist[x]->d_name, 1);
                 }
             continue;
             }
@@ -171,7 +213,7 @@ extern void linkfiles(file_t *files, const int linktype, const int only_current)
         }
 #ifdef UNICODE
         if (!M2W(dupelist[x]->d_name, wname)) {
-          fprintf(stderr, "error: MultiByteToWideChar failed: "); fwprint(stderr, dupelist[x]->d_name, 1);
+          mb2wc_failed(dupelist[x]->d_name);
           continue;
         }
 #endif /* UNICODE */
@@ -184,7 +226,8 @@ extern void linkfiles(file_t *files, const int linktype, const int only_current)
 #endif
         {
           fprintf(stderr, "warning: link target is a read-only file, not linking:\n-//-> ");
-          fwprint(stderr, dupelist[x]->d_name, 1);
+          jc_fwprint(stderr, dupelist[x]->d_name, 1);
+          exit_status = EXIT_FAILURE;
           continue;
         }
         /* Check file pairs for modification before linking */
@@ -192,14 +235,16 @@ extern void linkfiles(file_t *files, const int linktype, const int only_current)
         i = file_has_changed(srcfile);
         if (i) {
           fprintf(stderr, "warning: source file modified since scanned; changing source file:\n[SRC] ");
-          fwprint(stderr, dupelist[x]->d_name, 1);
+          jc_fwprint(stderr, dupelist[x]->d_name, 1);
           LOUD(fprintf(stderr, "file_has_changed: %d\n", i);)
           srcfile = dupelist[x];
+          exit_status = EXIT_FAILURE;
           continue;
         }
         if (file_has_changed(dupelist[x])) {
           fprintf(stderr, "warning: target file modified since scanned, not linking:\n-//-> ");
-          fwprint(stderr, dupelist[x]->d_name, 1);
+          jc_fwprint(stderr, dupelist[x]->d_name, 1);
+          exit_status = EXIT_FAILURE;
           continue;
         }
 #ifdef ON_WINDOWS
@@ -207,19 +252,22 @@ extern void linkfiles(file_t *files, const int linktype, const int only_current)
          * by skipping linking or changing the link source file as needed */
         if (STAT(srcfile->d_name, &s) != 0) {
           fprintf(stderr, "warning: win_stat() on source file failed, changing source file:\n[SRC] ");
-          fwprint(stderr, dupelist[x]->d_name, 1);
+          jc_fwprint(stderr, dupelist[x]->d_name, 1);
           srcfile = dupelist[x];
+          exit_status = EXIT_FAILURE;
           continue;
         }
         if (s.st_nlink >= 1024) {
           fprintf(stderr, "warning: maximum source link count reached, changing source file:\n[SRC] ");
           srcfile = dupelist[x];
+          exit_status = EXIT_FAILURE;
           continue;
         }
         if (STAT(dupelist[x]->d_name, &s) != 0) continue;
         if (s.st_nlink >= 1024) {
           fprintf(stderr, "warning: maximum destination link count reached, skipping:\n-//-> ");
-          fwprint(stderr, dupelist[x]->d_name, 1);
+          jc_fwprint(stderr, dupelist[x]->d_name, 1);
+          exit_status = EXIT_FAILURE;
           continue;
         }
 #endif
@@ -227,7 +275,8 @@ extern void linkfiles(file_t *files, const int linktype, const int only_current)
         if (linktype == 2) {
           if (STAT(dupelist[x]->d_name, &s) != 0) {
             fprintf(stderr, "warning: stat() on destination file failed, skipping:\n-##-> ");
-            fwprint(stderr, dupelist[x]->d_name, 1);
+            jc_fwprint(stderr, dupelist[x]->d_name, 1);
+            exit_status = EXIT_FAILURE;
             continue;
           }
 
@@ -252,7 +301,7 @@ extern void linkfiles(file_t *files, const int linktype, const int only_current)
         /* Rename the destination file to the temporary name */
 #ifdef UNICODE
         if (!M2W(tempname, wname2)) {
-          fprintf(stderr, "error: MultiByteToWideChar failed: "); fwprint(stderr, srcfile->d_name, 1);
+          mb2wc_failed(srcfile->d_name);
           continue;
         }
         i = MoveFileW(wname, wname2) ? 0 : 1;
@@ -261,7 +310,8 @@ extern void linkfiles(file_t *files, const int linktype, const int only_current)
 #endif
         if (i != 0) {
           fprintf(stderr, "warning: cannot move link target to a temporary name, not linking:\n-//-> ");
-          fwprint(stderr, dupelist[x]->d_name, 1);
+          jc_fwprint(stderr, dupelist[x]->d_name, 1);
+          exit_status = EXIT_FAILURE;
           /* Just in case the rename succeeded yet still returned an error, roll back the rename */
 #ifdef UNICODE
           MoveFileW(wname2, wname);
@@ -277,7 +327,7 @@ extern void linkfiles(file_t *files, const int linktype, const int only_current)
 #ifdef ON_WINDOWS
  #ifdef UNICODE
         if (!M2W(srcfile->d_name, wname2)) {
-          fprintf(stderr, "error: MultiByteToWideChar failed: "); fwprint(stderr, srcfile->d_name, 1);
+          mb2wc_failed(srcfile->d_name);
           continue;
         }
         if (CreateHardLinkW((LPCWSTR)wname, (LPCWSTR)wname2, NULL) == TRUE) success = 1;
@@ -299,28 +349,16 @@ extern void linkfiles(file_t *files, const int linktype, const int only_current)
                 /* chflags overrides the timestamps that were restored by copyfile, so we need to reapply those as well */
                 if (utimes(dupelist[x]->d_name, dupfile_original_tval) == 0) {
                   success = 1;
-                } else {
-                  fprintf(stderr, "warning: utimes() failed for destination file, reverting:\n-##-> ");
-                  fwprint(stderr, dupelist[x]->d_name, 1);
-                }
-              } else {
-                fprintf(stderr, "warning: chflags() failed for destination file, reverting:\n-##-> ");
-                fwprint(stderr, dupelist[x]->d_name, 1);
-              }
-            } else {
-              fprintf(stderr, "warning: copyfile() failed for destination file, reverting:\n-##-> ");
-              fwprint(stderr, dupelist[x]->d_name, 1);
-            }
-          } else {
-            fprintf(stderr, "warning: clonefile() failed for destination file, reverting:\n-##-> ");
-            fwprint(stderr, dupelist[x]->d_name, 1);
-          }
+                } else clonefile_error("utimes", dupelist[x]->d_name);
+              } else clonefile_error("chflags", dupelist[x]->d_name);
+            } else clonefile_error("copyfile", dupelist[x]->d_name);
+          } else clonefile_error("clonefile", dupelist[x]->d_name);
  #endif /* ENABLE_CLONEFILE_LINK */
         }
  #ifndef NO_SYMLINKS
         else {
-          i = make_relative_link_name(srcfile->d_name, dupelist[x]->d_name, rel_path);
-          LOUD(fprintf(stderr, "symlink GRN: %s to %s = %s\n", srcfile->d_name, dupelist[x]->d_name, rel_path));
+          i = jc_make_relative_link_name(srcfile->d_name, dupelist[x]->d_name, rel_path);
+          LOUD(fprintf(stderr, "symlink MRLN: %s to %s = %s\n", srcfile->d_name, dupelist[x]->d_name, rel_path));
           if (i < 0) {
             fprintf(stderr, "warning: make_relative_link_name() failed (%d)\n", i);
           } else if (i == 1) {
@@ -345,37 +383,34 @@ extern void linkfiles(file_t *files, const int linktype, const int only_current)
                 break;
 #endif
             }
-            fwprint(stdout, dupelist[x]->d_name, 1);
+            jc_fwprint(stdout, dupelist[x]->d_name, 1);
           }
         } else {
           /* The link failed. Warn the user and put the link target back */
+          exit_status = EXIT_FAILURE;
           if (!ISFLAG(flags, F_HIDEPROGRESS)) {
-            printf("-//-> "); fwprint(stdout, dupelist[x]->d_name, 1);
+            printf("-//-> "); jc_fwprint(stdout, dupelist[x]->d_name, 1);
           }
-          fprintf(stderr, "warning: unable to link '"); fwprint(stderr, dupelist[x]->d_name, 0);
-          fprintf(stderr, "' -> '"); fwprint(stderr, srcfile->d_name, 0);
+          fprintf(stderr, "warning: unable to link '"); jc_fwprint(stderr, dupelist[x]->d_name, 0);
+          fprintf(stderr, "' -> '"); jc_fwprint(stderr, srcfile->d_name, 0);
           fprintf(stderr, "': %s\n", strerror(errno));
 #ifdef UNICODE
           if (!M2W(tempname, wname2)) {
-            fprintf(stderr, "error: MultiByteToWideChar failed: "); fwprint(stderr, tempname, 1);
+            mb2wc_failed(tempname);
             continue;
           }
           i = MoveFileW(wname2, wname) ? 0 : 1;
 #else
           i = rename(tempname, dupelist[x]->d_name);
 #endif /* UNICODE */
-          if (i != 0) {
-            fprintf(stderr, "error: cannot rename temp file back to original\n");
-            fprintf(stderr, "original: "); fwprint(stderr, dupelist[x]->d_name, 1);
-            fprintf(stderr, "current:  "); fwprint(stderr, tempname, 1);
-          }
+          if (i != 0) revert_failed(dupelist[x]->d_name, tempname);
           continue;
         }
 
         /* Remove temporary file to clean up; if we can't, reverse the linking */
 #ifdef UNICODE
           if (!M2W(tempname, wname2)) {
-            fprintf(stderr, "error: MultiByteToWideChar failed: "); fwprint(stderr, tempname, 1);
+            mb2wc_failed(tempname);
             continue;
           }
         i = DeleteFileW(wname2) ? 0 : 1;
@@ -386,7 +421,8 @@ extern void linkfiles(file_t *files, const int linktype, const int only_current)
           /* If the temp file can't be deleted, there may be a permissions problem
            * so reverse the process and warn the user */
           fprintf(stderr, "\nwarning: can't delete temp file, reverting: ");
-          fwprint(stderr, tempname, 1);
+          jc_fwprint(stderr, tempname, 1);
+          exit_status = EXIT_FAILURE;
 #ifdef UNICODE
           i = DeleteFileW(wname) ? 0 : 1;
 #else
@@ -400,11 +436,7 @@ extern void linkfiles(file_t *files, const int linktype, const int only_current)
 #else
             i = rename(tempname, dupelist[x]->d_name);
 #endif
-            if (i != 0) {
-              fprintf(stderr, "\nwarning: couldn't revert the file to its original name\n");
-              fprintf(stderr, "original: "); fwprint(stderr, dupelist[x]->d_name, 1);
-              fprintf(stderr, "current:  "); fwprint(stderr, tempname, 1);
-            }
+            if (i != 0) revert_failed(dupelist[x]->d_name, tempname);
           }
         }
       }
@@ -414,7 +446,9 @@ extern void linkfiles(file_t *files, const int linktype, const int only_current)
     files = files->next;
   }
 
+  if (counter == 0) printf("%s", s_no_dupes);
+
   free(dupelist);
   return;
 }
-#endif /* both NO_HARDLINKS and NO_SYMLINKS */
+#endif /* NO_HARDLINKS + NO_SYMLINKS + !ENABLE_DEDUPE */
