@@ -7,7 +7,7 @@
 #include "jdupes.h"
 #include "act_summarize.h"
 
-extern void summarizematches(const file_t * restrict files)
+void summarizematches(const file_t * restrict files)
 {
   unsigned int numsets = 0;
   off_t numbytes = 0;
@@ -31,7 +31,7 @@ extern void summarizematches(const file_t * restrict files)
   }
 
   if (numsets == 0)
-    printf("No duplicates found.\n");
+    printf("%s", s_no_dupes);
   else
   {
     printf("%d duplicate files (in %d sets), occupying ", numfiles, numsets);
